"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestMMR = void 0;
var _assert = require("assert");
var _core = require("@kui-shell/core");
var Common = _interopRequireWildcard(require("./common"));
var CLI = _interopRequireWildcard(require("./cli"));
var ReplExpect = _interopRequireWildcard(require("./repl-expect"));
var SidecarExpect = _interopRequireWildcard(require("./sidecar-expect"));
var Selectors = _interopRequireWildcard(require("./selectors"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = void 0 && (void 0).__awaiter || function (thisArg, _arguments, P, generator) {
  function adopt(value) {
    return value instanceof P ? value : new P(function (resolve) {
      resolve(value);
    });
  }
  return new (P || (P = Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }
    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }
    function step(result) {
      result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
    }
    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
};
class TestMMR {
  /**
   * new TestMMR() instantiates a class of multi-model-response tests
   * @param { TestParam } param includes: command, testName and metadata
   * @param { string } command is the command needs to be executed
   * @param { string } testName (optional) helps with filtering the Mocha Test Suites by description
   * @param { string } metadata is the metadata shown in Sidecar
   */
  constructor(param) {
    this.param = param;
    this.testClickResult = (cmdIdx, command, expect) => app => __awaiter(this, void 0, void 0, function* () {
      yield CLI.expectPriorInput(Selectors.PROMPT_N(cmdIdx), command)(app);
      if (typeof expect === 'string') {
        yield ReplExpect.okWithString(expect, true)({
          app,
          count: cmdIdx
        });
      }
    });
  } // eslint-disable-line no-useless-constructor
  /**
   * name() starts a Mocha Test Suite
   * name() executes `command` in REPL and expects `prettyName` or `name` is shown in Sidecar
   */
  name(opt) {
    const {
      command,
      testName,
      metadata
    } = this.param;
    const testClickResult = this.testClickResult;
    describe(`mmr name ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      const {
        nameHash,
        prettyName,
        onclick
      } = opt;
      let cmdIdx = 0; // keep track of the command execution number
      const showName = prettyName || metadata.name;
      it(`should show name ${showName} ${nameHash ? ' and namehash ' + nameHash : ''} in sidecar header`, () => CLI.command(command, this.app).then(res => {
        cmdIdx = res.count;
        return res;
      }).then(ReplExpect.ok).then(SidecarExpect.open).then(res => __awaiter(this, void 0, void 0, function* () {
        yield SidecarExpect.name(res.count, showName);
        if (nameHash) {
          yield SidecarExpect.namehash(res.count, nameHash);
        }
      })).catch(Common.oops(this, true)));
      if (onclick && onclick.name) {
        it('should click the name part of sidecar and expect the command shows in repl', () => __awaiter(this, void 0, void 0, function* () {
          try {
            yield this.app.client.$(Selectors.SIDECAR_TITLE(cmdIdx)).then(_ => _.click());
            yield testClickResult(++cmdIdx, onclick.name.command, onclick.name.expect)(this.app);
          } catch (err) {
            yield Common.oops(this, true)(err);
          }
        }));
      }
      if (onclick && onclick.nameHash) {
        it('should click the namehash part of sidecar and expect the command shows in repl', () => __awaiter(this, void 0, void 0, function* () {
          try {
            yield this.app.client.$(Selectors.SIDECAR_ACTIVATION_TITLE(cmdIdx)).then(_ => _.click());
            yield testClickResult(++cmdIdx, onclick.nameHash.command, onclick.nameHash.expect)(this.app);
          } catch (err) {
            yield Common.oops(this, true)(err);
          }
        }));
      }
    });
  }
  /**
   * namespace() starts a Mocha Test Suite
   * namespace() executes `command` in REPL and expects `namespace` is shown in Sidecar
   *
   */
  namespace(opt) {
    const {
      command,
      testName,
      metadata
    } = this.param;
    const testClickResult = this.testClickResult;
    describe(`mmr namespace ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      const onclick = opt && opt.onclick;
      let cmdIdx = 0; // keep track of the command execution number
      it(`should show namespace ${metadata.namespace} in sidecar`, () => CLI.command(command, this.app).then(res => {
        cmdIdx = res.count;
        return res;
      }).then(ReplExpect.ok).then(SidecarExpect.open).then(SidecarExpect.namespace(metadata.namespace)).catch(Common.oops(this, true)));
      if (onclick) {
        it(`should click the namespace part of sidecar and expect the command shows in repl`, () => __awaiter(this, void 0, void 0, function* () {
          try {
            yield this.app.client.$(Selectors.SIDECAR_PACKAGE_NAME_TITLE(cmdIdx)).then(_ => _.click());
            yield testClickResult(++cmdIdx, onclick.command, onclick.expect)(this.app);
          } catch (err) {
            yield Common.oops(this, true)(err);
          }
        }));
      }
    });
  }
  /**
   * kind() starts a Mocha Test Suite
   * kind() executes `command` in REPL and expects `kind` is showin in Sidecar
   * @param { string } kind is the expected kind text shown in the Sidecar
   *
   */
  kind(kind) {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr kind ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      it(`should show kind ${kind} in sidecar`, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(SidecarExpect.kind(kind)).catch(Common.oops(this, true)));
    });
  }
  /**
   * badges() starts a Mocha Test Suite
   * badges() executes `command` in REPL and expects `badges` are showin in Sidecar
   * @param { BadgeSpec[] } badges is the expected badges shown in the Sidecar
   *
   */
  badges(badges) {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr badges ${testName || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      it(`should show badges in sidecar`, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(res => Promise.all(badges.map(badge => SidecarExpect.badge(badge.title, badge.css)(res)))));
    });
  }
  /**
   * modes() starts a Mocha Test Suite
   * modes() executes `command` in REPL and expects `modes` are showin in Sidecar
   * @param { MMRExpectMode[] } expectModes is the expected modes shown as Sidecar Tabs
   * @param { MMRExpectMode } defaultMode is the expected default Sidecar Tab
   *
   *
   */
  modes(expectModes, defaultMode) {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr modes ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      let cmdIdx;
      const showModes = () => {
        it(`should show sidecar tabs with command=${command}`, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(SidecarExpect.defaultMode(defaultMode)).then(SidecarExpect.modes(expectModes)).then(res => {
          cmdIdx = res.count;
          return res;
        }).catch(Common.oops(this, true)));
      };
      const clickToActivateTab = mode => __awaiter(this, void 0, void 0, function* () {
        const sel = Selectors.SIDECAR_MODE_BUTTON(cmdIdx, mode);
        yield this.app.client.$(sel).then(_ => _.waitForExist());
        yield this.app.client.execute(sel => {
          ;
          document.querySelector(sel).focus();
        }, sel);
        const elt = yield this.app.client.$(sel);
        (0, _assert.ok)(yield elt.isDisplayedInViewport());
        yield elt.click();
        yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON_SELECTED(cmdIdx, mode)).then(_ => _.waitForExist());
      });
      const cycleTheTabs = () => expectModes.forEach(expectMode => {
        it(`should switch to the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
          try {
            yield clickToActivateTab(expectMode.mode);
          } catch (err) {
            return Common.oops(this, true)(err);
          }
        }));
        if (expectMode.contentType === 'text/plain') {
          it(`should show plain text content via monaco in the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
            try {
              if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
                yield SidecarExpect.textPlainContentFromMonaco(expectMode.content)({
                  app: this.app,
                  count: cmdIdx
                });
              }
            } catch (err) {
              return Common.oops(this, true)(err);
            }
          }));
        } else if (expectMode.contentType === 'react') {
          it(`should show react content`, () => __awaiter(this, void 0, void 0, function* () {
            if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
              const selector = `${Selectors.SIDECAR_TAB_CONTENT(cmdIdx)} ${expectMode.selector}`;
              yield this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                const elt = yield this.app.client.$(selector);
                yield elt.waitForDisplayed();
                return expectMode.innerText === (yield elt.getText());
              }));
            }
          }));
        } else if (expectMode.contentType === 'table') {
          it(`should show ${expectMode.nRows} table rows in the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
            try {
              if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
                let idx = 0;
                yield this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                  const rows = yield this.app.client.$$(`${Selectors.SIDECAR_TAB_CONTENT(cmdIdx)} tbody tr`);
                  const actualRows = rows.length;
                  const expectedRows = expectMode.nRows;
                  if (++idx > 5) {
                    console.error(`still waiting for table rows actualRows=${actualRows} expectedRows=${expectedRows}`, `${Selectors.SIDECAR_TAB_CONTENT(cmdIdx)} tbody tr`);
                  }
                  return actualRows === expectedRows;
                }), {
                  timeout: CLI.waitTimeout
                });
              }
            } catch (err) {
              return Common.oops(this, true)(err);
            }
          }));
          it(`should show ${expectMode.nCells} table cells in the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
            try {
              if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
                yield this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                  const cells = yield this.app.client.$$(`${Selectors.SIDECAR_TAB_CONTENT(cmdIdx)} td`);
                  return cells.length === expectMode.nCells;
                }), {
                  timeout: CLI.waitTimeout
                });
              }
            } catch (err) {
              return Common.oops(this, true)(err);
            }
          }));
        } else if (expectMode.contentType === 'yaml') {
          if (expectMode.editor === true) {
            it(`should open editor and show yaml content via monaco in the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
              try {
                if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
                  yield SidecarExpect.yaml(expectMode.content)({
                    app: this.app,
                    count: cmdIdx
                  });
                }
              } catch (err) {
                return Common.oops(this, true)(err);
              }
            }));
          } else {
            it(`should show random content in the ${expectMode.mode} tab`, () => __awaiter(this, void 0, void 0, function* () {
              try {
                if (yield this.app.client.$(Selectors.SIDECAR_MODE_BUTTON(cmdIdx, expectMode.mode)).then(_ => _.isDisplayed())) {
                  yield SidecarExpect.textPlainContent(expectMode.content)({
                    app: this.app,
                    count: cmdIdx
                  });
                }
              } catch (err) {
                return Common.oops(this, true)(err);
              }
            }));
          }
        }
      });
      showModes();
      cycleTheTabs();
      cycleTheTabs();
    });
  }
  diffPlainText(mode, textB) {
    const {
      command,
      testName
    } = this.param;
    describe(`diff ${testName} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      it('should should open sidecar with diff view', () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(SidecarExpect.mode(mode)).then(SidecarExpect.textPlainContentFromMonaco(textB)).catch(Common.oops(this, true)));
    });
  }
  /**
   * toolbarButtons() starts a Mocha Test Suite
   * toolbarButtons() executes `command` and expects the `buttons` shown in Sidecar having correct labels and drildown handlers
   *
   * @param buttons is the expected array of `button` shown in the Sidecar Toolbar
   *
   */
  toolbarButtons(buttons) {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr toolbar buttons ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      let res;
      it(`should show toolbar buttons in sidecar `, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(appAndCount => {
        res = appAndCount;
        return appAndCount;
      }).then(res => Promise.all(buttons.map(button => SidecarExpect.button(button)(res)))).catch(Common.oops(this, true)));
      const drilldownButtons = buttons.filter(_ => _.kind === 'drilldown');
      if (drilldownButtons.length > 0) {
        it(`should drilldown toolbar buttons in sidecar`, () => __awaiter(this, void 0, void 0, function* () {
          try {
            const {
              app,
              count
            } = yield CLI.command(command, this.app);
            yield ReplExpect.ok({
              app,
              count
            });
            yield SidecarExpect.open({
              app,
              count
            });
            yield _core.Util.promiseEach(drilldownButtons, (button, index) => __awaiter(this, void 0, void 0, function* () {
              // the button should be clickable
              const buttonSelector = Selectors.SIDECAR_TOOLBAR_BUTTON(res.count, button.mode);
              const buttonElt = yield app.client.$(buttonSelector);
              yield CLI.waitForRepl(this.app);
              yield buttonElt.waitForDisplayed();
              yield buttonElt.click();
              if (button.confirm) {
                // const dialog = Selectors.CONFIRM_DIALOG
                const denyItSel = Selectors.CONFIRM_DIALOG_CANCEL_BUTTON;
                const confirmItSel = Selectors.CONFIRM_DIALOG_CONFIRM_BUTTON;
                const [denyIt, confirmIt] = yield Promise.all([app.client.$(denyItSel), app.client.$(confirmItSel)]);
                yield Promise.all([denyIt, confirmIt].map(_ => _.waitForDisplayed()));
                // first click deny, and expect the confirm dialog to be gone
                yield denyIt.click();
                yield denyIt.waitForExist({
                  timeout: 5000,
                  reverse: true
                });
                // after clicking deny, the next prompt should *not* exist
                const nextPromptSelector = Selectors.PROMPT_N(count + 1 + index + 1);
                yield app.client.$(nextPromptSelector).then(_ => _.waitForExist({
                  timeout: 5000,
                  reverse: true
                }));
                // now click the button again, then click confirm
                yield buttonElt.click();
                const [denyIt2, confirmIt2] = yield Promise.all([app.client.$(denyItSel), app.client.$(confirmItSel)]);
                yield Promise.all([denyIt2, confirmIt2].map(_ => _.waitForDisplayed()));
                yield confirmIt2.click();
              }
              // after clicking the button, a command should show up in the next prompt
              const promptSelector = Selectors.PROMPT_N(count + 1 + index);
              if (!button.expectError) {
                yield ReplExpect.ok({
                  app,
                  count: count + 1 + index
                });
              } else {
                yield ReplExpect.error(button.expectError);
              }
              if (typeof button.command === 'string') {
                yield CLI.expectPriorInput(promptSelector, button.command)(app);
              }
            }));
          } catch (err) {
            yield Common.oops(this, true)(err);
          }
        }));
      }
    });
  }
  /**
   * toolbarText() starts a Mocha Test Suite
   * toolbarText() executes `command` and expects Sidecar Toolbar has correct `text` and `type`
   *
   * @param  toolbarText is the expected text content and type shown in the Sidecar Toolbar
   */
  toolbarText(toolbarText) {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr toolbar text ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      it(`should show toolbar text in sidecar `, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(SidecarExpect.toolbarText(toolbarText)).catch(Common.oops(this, true)));
    });
  }
  toolbarNotExist() {
    const {
      command,
      testName
    } = this.param;
    describe(`mmr toolbar ${testName || command || ''} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      it(`should not show toolbar in sidecar `, () => CLI.command(command, this.app).then(ReplExpect.ok).then(SidecarExpect.open).then(res => this.app.client.$(Selectors.SIDECAR_TOOLBAR(res.count)).then(_ => _.waitForExist({
        timeout: CLI.waitTimeout,
        reverse: true
      }))).catch(Common.oops(this, true)));
    });
  }
}
exports.TestMMR = TestMMR;