"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.blankWithOpts = exports.blank = void 0;
exports.blockAfter = blockAfter;
exports.blockCount = blockCount;
exports.comment = comment;
exports.consoleToBeClear = void 0;
exports.elsewhere = elsewhere;
exports.okWithTextContent = exports.okWithStringEventually = exports.okWithString = exports.okWithStreamingOutput = exports.okWithPtyOutputEventually = exports.okWithPtyOutput = exports.okWithOnly = exports.okWithEvents = exports.okWithDropDownList = exports.okWithCustom = exports.okWithAny = exports.okWith = exports.ok = exports.justOK = exports.error = void 0;
exports.splitCount = splitCount;
exports.tableWithNRows = tableWithNRows;
var assert = _interopRequireWildcard(require("assert"));
var _cli = require("./cli");
var Selectors = _interopRequireWildcard(require("./selectors"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = void 0 && (void 0).__awaiter || function (thisArg, _arguments, P, generator) {
  function adopt(value) {
    return value instanceof P ? value : new P(function (resolve) {
      resolve(value);
    });
  }
  return new (P || (P = Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }
    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }
    function step(result) {
      result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
    }
    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
};
function expectOK(appAndCount, opt) {
  return __awaiter(this, void 0, void 0, function* () {
    // appAndCount.count is the prompt index of this command... so +1 gives us the next prompt, whose existence signals that this command has finished
    const app = appAndCount.app;
    const N = appAndCount.count + 1;
    const {
      splitIndex = 1
    } = appAndCount;
    const nextPrompt = process.env.KUI_POPUP ? Selectors.STATUS_STRIPE_PROMPT : !process.env.BOTTOM_INPUT_MODE ? Selectors.PROMPT_N_FOR_SPLIT(N, splitIndex) : Selectors.BOTTOM_PROMPT;
    const nextPromptDom = yield app.client.$(nextPrompt);
    yield nextPromptDom.waitForExist({
      timeout: _cli.waitTimeout
    }); // wait for the next prompt to appear
    const promptValue = yield nextPromptDom.getValue();
    if (!process.env.BOTTOM_INPUT_MODE && (!opt || !opt.nonBlankPromptOk) && promptValue && promptValue.length !== 0) {
      console.error(`Expected prompt value to be empty: ${promptValue}`);
      if (promptValue) {
        assert.strictEqual(promptValue.length, 0);
      }
    }
    if (opt && opt.expectError) return false;
    const html = yield app.client.$(Selectors.OK_N(N - 1, splitIndex)).then(_ => _.getHTML());
    const okReg = new RegExp(process.env.OK) || /ok/;
    assert.ok(okReg.test(html)); // make sure it says "ok" !
    return Promise.resolve().then(() => __awaiter(this, void 0, void 0, function* () {
      // validate any expected list entry
      if (opt && opt.expectString) {
        // expect exactly one entry
        yield app.client.$(`${Selectors.LIST_RESULTS_BY_NAME_N(N - 1, splitIndex)} .entity-name`).then(_ => _.getText()).then(name => assert.strictEqual(name, opt.expectString));
      } else if (opt && opt.expectArray) {
        // expect several entries, of which opt is one // NOTE: what does it mean by opt is one???
        yield app.client.$$(Selectors.LIST_RESULTS_BY_NAME_N(N - 1, splitIndex)).then(names => Promise.all(names.map(_ => _.getText()))).then(names => assert.notStrictEqual(names.findIndex(_ => _.indexOf(opt.expectArray[0]) >= 0), -1, `name="${opt.expectArray[0]}" not found in list=${names}`));
      } else if (opt && (opt.selector || opt.expect)) {
        // more custom, look for expect text under given selector
        const selector = `${opt.streaming ? Selectors.OUTPUT_N_STREAMING(N - 1, splitIndex) : Selectors.OUTPUT_N(N - 1, splitIndex)}${opt.selfSelector || ''} ${opt.selector || ''}`;
        if (opt.elements) {
          return app.client.$$(selector);
        } else {
          let idx = 0;
          return app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
            const txt = yield app.client.$(selector).then(_ => __awaiter(this, void 0, void 0, function* () {
              yield _.waitForExist({
                timeout: _cli.waitTimeout
              });
              return _.getText();
            }));
            if (++idx > 5) {
              console.error(`still waiting for expected text actualText=${txt} expectedText=${opt.expect}`);
            }
            if (opt.exact) return txt === opt.expect;else if (opt.expect) {
              if (txt.indexOf(opt.expect) < 0) {
                console.error(`Expected string not found expected=${opt.expect} idx=${txt.indexOf(opt.expect)} actual=${txt}`);
                return txt.indexOf(opt.expect) >= 0;
              }
            }
            return true;
          }), {
            timeout: _cli.waitTimeout
          }).then(() => {
            return opt.passthrough ? N - 1 : selector; // so that the caller can inspect the selector in more detail
          }).then(res => res);
        }
      } else if (opt && opt.expectJustOK === true) {
        // ensure that there is nothing other than "ok"
        yield app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
          const txt = yield app.client.$(Selectors.OUTPUT_N(N - 1, splitIndex)).then(_ => _.getText());
          const justOK = process.env.OK || 'ok';
          return txt.length === 0 || txt === justOK;
        }), {
          timeout: _cli.waitTimeout
        });
      } else {
        // nothing to validate with the "console" results of the command
        // return the index of the last executed command
        return N - 1;
      }
    })).then(res => opt && (opt.selector || opt.passthrough) ? res : app) // return res rather than app, if requested
    .then(res => res);
  });
}
const ok = res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, {
    passthrough: true
  }).then(N => res.app.client.$$(Selectors.LIST_RESULTS_BY_NAME_N(N, res.splitIndex))).then(elts => assert.strictEqual(elts.length, 0)).then(() => res);
});
exports.ok = ok;
const error = (statusCode, expect) => res => __awaiter(void 0, void 0, void 0, function* () {
  yield expectOK(res, {
    expectError: true
  });
  if (expect) {
    const selfSelector = `.oops[data-status-code="${statusCode || 0}"]`;
    const selector = `${Selectors.OUTPUT_N(res.count, res.splitIndex || 1)}${selfSelector || ''}`;
    let idx = 0;
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
      const actualText = yield res.app.client.$(selector).then(_ => _.getText());
      if (++idx > 5) {
        console.error(`still waiting for actualText=${actualText} expectedText=${expect}`);
      }
      return actualText.indexOf(expect) >= 0;
    }), {
      timeout: _cli.waitTimeout
    });
  }
  return res.app;
});
exports.error = error;
const blankWithOpts = (opts = {}) => res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, Object.assign({
    selector: '',
    expectError: true
  }, opts));
});
exports.blankWithOpts = blankWithOpts;
const blank = res => blankWithOpts()(res);
/** The return type `any` comes from webdriverio waitUntil */
exports.blank = blank;
const consoleToBeClear = (app, residualBlockCount = 1, splitIndex = 1) => {
  let idx = 0;
  return app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
    const actualBlockCount = (yield app.client.$$(Selectors.PROMPT_BLOCK_FOR_SPLIT(splitIndex))).length;
    if (++idx > 5) {
      console.error(`still waiting for residualBlockCount=${residualBlockCount}; actualBlockCount=${actualBlockCount}`);
    }
    return actualBlockCount === residualBlockCount;
  }), {
    timeout: _cli.waitTimeout
  });
};
/** as long as its ok, accept anything */
exports.consoleToBeClear = consoleToBeClear;
const okWithCustom = custom => res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, custom);
});
exports.okWithCustom = okWithCustom;
const okWithTextContent = (expect, exact = false, failFast = true, sel = ' ') => res => __awaiter(void 0, void 0, void 0, function* () {
  // Notes: webdriverio's getText seems to use .innerText to extract
  // the text from a given selector; this is quite unreliable in
  // terms of whitespace preservation; e.g. <div><span>
  // </span><span> </span></div> will preserve whitespace, but if
  // the inner spans have are inline-block, then innerText will not
  // preserve whitespace; textContent *will* preserve whitespace
  const selector = yield okWithCustom({
    selector: sel
  })(res);
  const txt = yield (0, _cli.getTextContent)(res.app, selector);
  if (exact) {
    assert.strictEqual(txt, expect);
    return true;
  } else {
    if (txt.indexOf(expect) < 0) {
      console.error(`Expected string not found expected=${expect} idx=${txt.indexOf(expect)} actual=${txt}`);
      if (failFast) {
        assert.ok(txt.indexOf(expect) >= 0);
        return true;
      } else {
        return false;
      }
    }
  }
});
exports.okWithTextContent = okWithTextContent;
const okWithString = (expect, exact = false, streaming = false) => res => __awaiter(void 0, void 0, void 0, function* () {
  // first try innerText
  return okWithCustom({
    expect,
    exact,
    streaming
  })(res).catch(err1 => __awaiter(void 0, void 0, void 0, function* () {
    // use .textContent as a backup plan
    return okWithTextContent(expect, exact)(res).catch(() => {
      throw err1;
    });
  }));
});
exports.okWithString = okWithString;
const okWithStreamingOutput = (expect, exact = false) => okWithString(expect, exact, true);
exports.okWithStreamingOutput = okWithStreamingOutput;
const okWithPtyOutput = (expect, exact = false) => okWithString(expect, exact);
exports.okWithPtyOutput = okWithPtyOutput;
const okWithStringEventually = (expect, exact = false) => res => {
  return res.app.client.waitUntil(() => {
    try {
      return okWithString(expect, exact)(res);
    } catch (err) {
      // swallow
    }
  }, {
    timeout: _cli.waitTimeout
  });
};
exports.okWithStringEventually = okWithStringEventually;
const okWithPtyOutputEventually = (expect, exact = false) => res => {
  return res.app.client.waitUntil(() => {
    try {
      return okWithPtyOutput(expect, exact)(res);
    } catch (err) {
      // swallow
    }
  }, {
    timeout: _cli.waitTimeout
  });
};
exports.okWithPtyOutputEventually = okWithPtyOutputEventually;
const okWithDropDownList = (openAndExpectLabel, click = false, closeAfterOpen = true) => res => __awaiter(void 0, void 0, void 0, function* () {
  const toggler = yield okWithCustom({
    selector: Selectors.DROPDOWN
  })(res);
  return res.app.client.$(toggler).then(_ => _.click()).then(() => res.app.client.$(Selectors.DROPDOWN_N_MENU_ITEM(res.count, openAndExpectLabel, res.splitIndex))).then(_ => __awaiter(void 0, void 0, void 0, function* () {
    if (click) {
      yield _.waitForDisplayed();
      yield _.click();
    }
    if (closeAfterOpen) {
      yield res.app.client.$(toggler).then(_ => _.click());
    }
    return res;
  }));
});
exports.okWithDropDownList = okWithDropDownList;
const currentEventCount = (app, outputCount) => __awaiter(void 0, void 0, void 0, function* () {
  const events = yield app.client.$$(Selectors.TABLE_FOOTER(outputCount));
  const res = !events ? 0 : events.length;
  return res;
});
const okWithEvents = (ctx, res) => __awaiter(void 0, void 0, void 0, function* () {
  let idx = 0;
  yield ctx.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
    const actualEventCount = yield currentEventCount(ctx.app, res.count);
    if (++idx > 5) {
      console.error('still waiting for events');
    }
    console.log('actualEventCount', actualEventCount);
    return actualEventCount > 0;
  }), {
    timeout: _cli.waitTimeout
  });
  return res;
});
/** as long as its ok, accept anything */
exports.okWithEvents = okWithEvents;
const okWithAny = res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res);
});
/** expect ok and *only* the given result value */
exports.okWithAny = okWithAny;
const okWithOnly = entityName => res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, {
    expectString: entityName
  });
});
/** expect ok and at least the given result value */
// FIXME: okWithAtLeast??
exports.okWithOnly = okWithOnly;
const okWith = entityName => res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, {
    expectArray: [entityName]
  });
});
/** expect just ok, and no result value */
exports.okWith = okWith;
const justOK = res => __awaiter(void 0, void 0, void 0, function* () {
  return expectOK(res, {
    expectJustOK: true
  }).then(() => res);
});
/** Expect the given number of terminal splits in the current tab, and check whether the last split has inverse colors */
exports.justOK = justOK;
function splitCount(expectedSplitCount, inverseColors = false, expectedSplitIndex) {
  return app => __awaiter(this, void 0, void 0, function* () {
    let idx = 0;
    yield app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
      const splits = yield app.client.$$(Selectors.SPLITS);
      if (++idx > 5) {
        console.error(`still waiting for splitCount; actual=${splits.length} expected=${expectedSplitCount}`);
      }
      return splits.length === expectedSplitCount;
    }), {
      timeout: _cli.waitTimeout
    });
    if (inverseColors) {
      yield app.client.$(Selectors.SPLIT_N(expectedSplitIndex || expectedSplitCount, inverseColors)).then(_ => _.waitForExist({
        timeout: _cli.waitTimeout
      }));
    }
  });
}
/** Expect table with N rows */
function tableWithNRows(N) {
  return res => {
    return res.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
      const rows = yield res.app.client.$$(Selectors.LIST_RESULTS_N(res.count));
      return rows.length === N;
    }), {
      timeout: _cli.waitTimeout
    });
  };
}
/** Expect an ElsewhereCommentaryResponse */
function elsewhere(expectedBody, N) {
  return res => __awaiter(this, void 0, void 0, function* () {
    let idx = 0;
    yield res.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
      const actualBody = yield res.app.client.$(Selectors.OUTPUT_N(res.count, N === undefined ? res.splitIndex : N)).then(_ => _.getText());
      if (++idx > 5) {
        console.error(`still waiting for body; actual=${actualBody}; expected=${expectedBody}`);
      }
      return actualBody === expectedBody;
    }), {
      timeout: _cli.waitTimeout
    });
    return res.app;
  });
}
/** Expect a CommentaryResponse */
function comment(expectedBody, expectedTitle) {
  return res => __awaiter(this, void 0, void 0, function* () {
    const output = Selectors.OUTPUT_N(res.count);
    if (expectedTitle) {
      yield res.app.client.$(`${output} ${Selectors.TERMINAL_CARD}`).then(_ => _.waitForDisplayed({
        timeout: _cli.waitTimeout
      }));
      const actualTitle = yield res.app.client.$(`${output} ${Selectors.TERMINAL_CARD_TITLE}`).then(_ => _.getText());
      assert.strictEqual(actualTitle, expectedTitle);
    }
    const actualBody = yield res.app.client.$(`${output} ${Selectors.TERMINAL_CARD_BODY}`).then(_ => _.getText());
    assert.strictEqual(actualBody, expectedBody);
    return res;
  });
}
/** Verify that the number of blocks equals the expected count */
function blockCount() {
  return {
    inSplit: splitIndex => ({
      is: expected => {
        it(`should have expectedBlockCount=${typeof expected === 'number' ? expected : '...'} inSplit=${splitIndex}`, () => {
          let idx = 0;
          const expectedBlockCount = typeof expected === 'number' ? expected : expected();
          return this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
            const blocks = yield this.app.client.$$(Selectors.PROMPT_BLOCK_FOR_SPLIT(splitIndex));
            const actualBlockCount = blocks.length;
            if (++idx > 5) {
              console.error(`still waiting for expectedBlockCount=${expectedBlockCount}; actualBlockCount=${actualBlockCount}`, blocks);
            }
            return actualBlockCount === expectedBlockCount;
          }), {
            timeout: _cli.waitTimeout
          });
        });
      }
    })
  };
}
/** Transform the given block finder to one that can find the next block */
function blockAfter(res, delta = 1) {
  return Object.assign({}, res, {
    count: res.count + delta
  });
}