"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TestTable = void 0;
var assert = _interopRequireWildcard(require("assert"));
var _core = require("@kui-shell/core");
var Common = _interopRequireWildcard(require("./common"));
var CLI = _interopRequireWildcard(require("./cli"));
var ReplExpect = _interopRequireWildcard(require("./repl-expect"));
var Selectors = _interopRequireWildcard(require("./selectors"));
var Utils = _interopRequireWildcard(require("./util"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = void 0 && (void 0).__awaiter || function (thisArg, _arguments, P, generator) {
  function adopt(value) {
    return value instanceof P ? value : new P(function (resolve) {
      resolve(value);
    });
  }
  return new (P || (P = Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }
    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }
    function step(result) {
      result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
    }
    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
};
class TestTable {
  constructor(testName, tests) {
    this.testName = testName || 'should test table';
    this.tests = tests;
  }
  /** new TestTable().run() will start a mocha test suite */
  run() {
    const self = this; // eslint-disable-line @typescript-eslint/no-this-alias
    describe(`${this.testName} ${process.env.MOCHA_RUN_TARGET || ''}`, function () {
      before(Common.before(this));
      after(Common.after(this));
      self.ctx = this;
      // execute the tests accordingly
      if (self.tests) {
        const {
          exec,
          drilldown,
          status
        } = self.tests;
        if (exec) {
          self.executeAndValidate(exec.command, exec.expectTable, exec.validation);
        }
        if (drilldown) {
          self.drilldownFromTable(drilldown.expectTable);
        }
        if (status) {
          self.evalStatus(status);
        }
      }
    });
  }
  evalStatus(_) {
    const {
      expectRow,
      command,
      statusDescription,
      ctx = this.ctx
    } = _;
    const self = this; // eslint-disable-line @typescript-eslint/no-this-alias
    it(`${statusDescription || 'should show status badge in row'}`, () => __awaiter(this, void 0, void 0, function* () {
      try {
        const count = command ? yield CLI.command(command, ctx.app).then(_ => _.count) : self.outputCount;
        yield _core.Util.promiseEach(expectRow, row => __awaiter(this, void 0, void 0, function* () {
          const rowSelector = `${Selectors.OUTPUT_N(count)} ${Selectors.BY_NAME(row.name)}`;
          yield (yield ctx.app.client.$(rowSelector)).waitForExist();
          // wait for message
          const messageSelector = `${Selectors.OUTPUT_N(count)} ${Selectors.TABLE_CELL(row.name, 'MESSAGE')}`;
          yield (yield ctx.app.client.$(messageSelector)).waitForExist();
          yield Utils.expectText(ctx.app, row.message)(messageSelector);
          // wait for badge
          const badge = `${rowSelector} [data-tag="badge"].${row.badgeCss}`;
          yield (yield ctx.app.client.$(badge)).waitForExist();
          yield Utils.expectText(ctx.app, row.badgeText)(badge);
        }));
      } catch (err) {
        yield Common.oops(ctx, true)(err);
      }
    }));
  }
  /**
   * Execute the table-generating command, and validate the content.
   *
   */
  executeAndValidate(command, expectTable, validation, _ctx) {
    const ctx = this.ctx || _ctx;
    const self = this; // eslint-disable-line @typescript-eslint/no-this-alias
    it(`should execute command from test table: ${command}`, () => CLI.command(command, ctx.app).then(res => {
      self.cmdIdx = res.count;
      return res;
    }).then(res => __awaiter(this, void 0, void 0, function* () {
      yield res.app.client.$(Selectors.TABLE_TITLE(res.count - 1)).then(_ => _.waitForExist({
        timeout: CLI.waitTimeout,
        reverse: true
      }));
      if (validation) {
        if (validation.hasGridButton) {
          yield res.app.client.$(Selectors.TABLE_SHOW_AS_GRID(res.count)).then(_ => _.waitForDisplayed());
        }
        if (validation.asGrid) {
          yield res.app.client.$(Selectors.TABLE_AS_GRID(res.count)).then(_ => _.waitForDisplayed());
        }
        if (validation.hasSequenceButton) {
          yield res.app.client.$(Selectors.TABLE_SHOW_AS_SEQUENCE(res.count)).then(_ => _.waitForDisplayed());
        }
        if (validation.switchToSequence) {
          yield res.app.client.$(Selectors.TABLE_SHOW_AS_SEQUENCE(res.count)).then(_ => __awaiter(this, void 0, void 0, function* () {
            yield _.waitForDisplayed();
            yield _.click();
          }));
          yield res.app.client.$(Selectors.TABLE_AS_SEQUENCE(res.count)).then(_ => _.waitForDisplayed());
          if (validation.bars) {
            yield _core.Util.promiseEach(validation.bars, width => __awaiter(this, void 0, void 0, function* () {
              res.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                return res.app.client.$(Selectors.TABLE_AS_SEQUENCE_BAR_WIDTH(res.count, width)).then(_ => _.isDisplayed());
              }));
            }));
          }
        }
        if (validation.switchToList) {
          yield res.app.client.$(Selectors.TABLE_SHOW_AS_LIST(res.count)).then(_ => __awaiter(this, void 0, void 0, function* () {
            yield _.waitForDisplayed();
            yield _.click();
          }));
          yield res.app.client.$(Selectors.TABLE_AS_LIST(res.count)).then(_ => _.waitForDisplayed());
        }
      }
      return res;
    })).then(ReplExpect.okWithCustom({
      selector: Selectors.TABLE_HEADER_CELL(expectTable.header.name)
    })).catch(Common.oops(ctx, true)));
    if (validation) {
      if (validation.cells) {
        expectTable.body.forEach((_, rowIdx) => {
          it(`should validate cells of row ${rowIdx} in test table output: ${command}`, () => __awaiter(this, void 0, void 0, function* () {
            const cellSelector = `${Selectors.OUTPUT_LAST} tbody tr:nth-child(${rowIdx + 1}) td > .cell-inner`;
            const cells = yield ctx.app.client.$$(cellSelector);
            if (cells.length > 1) {
              ;
              (yield Promise.all(cells.map(_ => _.getText()))).forEach((_, cellIdx) => validation.cells[cellIdx](_, rowIdx));
            } else if (validation.cells.length === 1 && cells.length === 1) {
              // got one cell, expecting one cell
              validation.cells[0](yield cells[0].getText(), rowIdx);
            } else {
              const actualCellCount = cells.length;
              assert.fail(`mismatch between expected cell count ${validation.cells.length} and actual cell count ${actualCellCount}`);
            }
          }));
        });
      }
    }
  }
  /**
   * drilldownFromTable() drilldowns from the table
   *
   * @param { Table } expectTable is the expected table shown in the REPL
   *
   */
  drilldownFromTable(expectTable, _ctx) {
    const ctx = _ctx || this.ctx;
    const self = this; // eslint-disable-line @typescript-eslint/no-this-alias
    const clickCell = (row, command) => {
      it(`should click to execute from test table: ${command}`, () => __awaiter(this, void 0, void 0, function* () {
        try {
          // work around tooltips not going away when clicking on
          // consecutive/nearby table rows in sequence
          yield ctx.app.client.$(Selectors.STATUS_STRIPE).then(_ => _.moveTo());
          yield new Promise(resolve => setTimeout(resolve, 1000));
          const cell = yield ctx.app.client.$(`${Selectors.OUTPUT_N(self.cmdIdx)} ${Selectors.TABLE_CELL(row.name, expectTable.header.name)}`);
          yield cell.waitForExist({
            timeout: 10000
          });
          yield cell.scrollIntoView();
          yield cell.moveTo();
          yield cell.waitForClickable({
            timeout: 10000
          });
          yield cell.click();
          // expect 2 splits in total
          yield ReplExpect.splitCount(2)(ctx.app);
        } catch (err) {
          yield Common.oops(ctx, true)(err);
        }
      }));
    };
    // For each row, check the first cell
    expectTable.body.forEach(row => {
      clickCell(row, row.onclick);
    });
  }
}
exports.TestTable = TestTable;