/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as Selectors from './selectors';
import { keys } from './keys';
export const timeout = Math.max(5000, parseInt(process.env.TIMEOUT) || 60000);
export const waitTimeout = timeout - 5000;
export const waitTimeoutOption = { timeout: waitTimeout };
/** grab focus for the repl */
export const grabFocus = (app, currentPromptBlock = process.env.KUI_POPUP
    ? Selectors.STATUS_STRIPE_BLOCK
    : !process.env.BOTTOM_INPUT_MODE
        ? Selectors.CURRENT_PROMPT_BLOCK
        : Selectors.BOTTOM_PROMPT_BLOCK, currentPrompt = process.env.KUI_POPUP
    ? Selectors.STATUS_STRIPE_PROMPT
    : !process.env.BOTTOM_INPUT_MODE
        ? Selectors.CURRENT_PROMPT
        : Selectors.BOTTOM_PROMPT) => __awaiter(void 0, void 0, void 0, function* () {
    return app.client
        .$(currentPrompt)
        .then(_ => _.moveTo()
        .catch(() => true)
        .then(() => _.click()))
        .then(() => app.client.$(currentPromptBlock))
        .then(_ => _.waitForEnabled(waitTimeoutOption))
        .catch(err => {
        console.error(err);
        // probably ok, we are doing this is just in case it helps
    });
});
/**
 * Execute a CLI command, and return the data-input-count of that command
 *
 */
export const command = (cmd, app, noNewline = false, noCopyPaste = process.env.TRAVIS_OS_NAME === 'osx', noFocus = false, block = process.env.KUI_POPUP
    ? Selectors.STATUS_STRIPE_BLOCK
    : process.env.BOTTOM_INPUT_MODE
        ? Selectors.CURRENT_PROMPT_BLOCK
        : Selectors.CURRENT_PROMPT_BLOCK_FOR_SPLIT(1), currentPrompt = process.env.KUI_POPUP
    ? Selectors.STATUS_STRIPE_PROMPT
    : !process.env.BOTTOM_INPUT_MODE
        ? Selectors.CURRENT_PROMPT_FOR_SPLIT(1)
        : Selectors.BOTTOM_PROMPT) => __awaiter(void 0, void 0, void 0, function* () {
    return app.client
        .$(block)
        .then(_ => _.waitForExist(waitTimeoutOption))
        .then(() => __awaiter(void 0, void 0, void 0, function* () {
        if (process.env.BOTTOM_INPUT_MODE)
            yield app.client.$(Selectors.BOTTOM_PROMPT_BLOCK).then(_ => _.waitForExist(waitTimeoutOption));
        if (!noFocus)
            return grabFocus(app, block, currentPrompt);
    }))
        .then(() => app.client.$(process.env.BOTTOM_INPUT_MODE ? Selectors.BOTTOM_PROMPT_BLOCK : block))
        .then(_ => _.getAttribute('data-input-count'))
        .then((count) => __awaiter(void 0, void 0, void 0, function* () {
        if (!noCopyPaste && cmd.length > 1) {
            // use the clipboard for a fast path
            yield app.client.execute(text => navigator.clipboard.writeText(text).then(() => document.execCommand('paste')), cmd);
        }
        else {
            // slow path
            const currentValue = yield app.client.$(currentPrompt).then(_ => _.getValue());
            const doThis = `${currentValue}${cmd}`;
            yield app.client.$(currentPrompt).then(_ => _.setValue(doThis));
        }
        if (noNewline !== true)
            yield app.client.keys(keys.ENTER);
        return { app: app, count: parseInt(count) };
    }));
});
/** Execute the given command in the given split */
export const commandInSplit = (cmd, app, splitIndex) => __awaiter(void 0, void 0, void 0, function* () {
    const resp = yield command(cmd, app, undefined, undefined, undefined, Selectors.CURRENT_PROMPT_BLOCK_FOR_SPLIT(splitIndex), Selectors.CURRENT_PROMPT_FOR_SPLIT(splitIndex));
    return Object.assign(resp, { splitIndex });
});
export const paste = (cmd, app, nLines = 1) => __awaiter(void 0, void 0, void 0, function* () {
    return app.client
        .$(Selectors.CURRENT_PROMPT_BLOCK)
        .then(_ => _.waitForExist(waitTimeoutOption))
        .then(() => app.client.$(Selectors.CURRENT_PROMPT_BLOCK))
        .then(_ => _.getAttribute('data-input-count'))
        .then((count) => __awaiter(void 0, void 0, void 0, function* () {
        app.electron.clipboard.writeText(cmd);
        yield app.client.execute(() => document.execCommand('paste'));
        return { app: app, count: parseInt(count) + nLines - 1 };
    }));
});
/** wait for the repl to be active */
export const waitForRepl = (app) => __awaiter(void 0, void 0, void 0, function* () {
    if (process.env.KUI_POPUP) {
        yield app.client.$(Selectors.STATUS_STRIPE_PROMPT).then(_ => _.waitForEnabled(waitTimeoutOption));
    }
    else {
        const prompt = yield app.client.$(Selectors.CURRENT_PROMPT);
        yield prompt.scrollIntoView();
        yield prompt.waitForEnabled(waitTimeoutOption);
    }
    return app;
});
/**
 * Wait, if needed, for a proxy session
 *
 */
export const waitForSession = (ctx, noProxySessionWait = false) => __awaiter(void 0, void 0, void 0, function* () {
    if (!process.env.MOCHA_RUN_TARGET || process.env.MOCHA_RUN_TARGET === 'electron') {
        yield ctx.app.client.waitUntilWindowLoaded();
    }
    if (process.env.MOCHA_RUN_TARGET === 'webpack' && process.env.KUI_USE_PROXY === 'true' && !noProxySessionWait) {
        // wait for the proxy session to be established
        try {
            yield ctx.app.client
                .$(`${Selectors.CURRENT_TAB}.kui--session-init-done`)
                .then(_ => _.waitForExist(waitTimeoutOption));
            yield ctx.app.client.$(Selectors.WELCOME_BLOCK).then(_ => _.waitForDisplayed(waitTimeoutOption));
        }
        catch (err) {
            throw new Error('error waiting for proxy session init');
        }
    }
});
export const getTextContent = (app, selector) => __awaiter(void 0, void 0, void 0, function* () {
    return app.client.execute(selector => {
        try {
            return document.querySelector(selector).textContent;
        }
        catch (err) {
            console.error('error in getTextContent', err);
            // intentionally returning undefined
        }
    }, selector);
});
/** wait for the result of a cli.command */
export const makeCustom = (selector, expect, exact) => ({
    selector: selector,
    expect: expect,
    exact: exact
});
/**
 * Exit code code for the given http status code; this is an identity function; for headless mode, there is the -256 part.
 * See headless.js for the analogous headless implementation.
 */
export const exitCode = (statusCode) => statusCode;
export const expectInput = (selector, expectedText) => (app) => __awaiter(void 0, void 0, void 0, function* () {
    yield app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const inputText = yield app.client.$(selector).then(_ => _.getValue());
        return inputText === expectedText;
    }), waitTimeoutOption);
    return app;
});
/** text repl input context text */
export function expectInputContext(res, N, expectedText, exact = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const text = yield res.app.client.$(Selectors.PROMPT_CONTEXT_N(N)).then(_ => _.getText());
        return exact ? expectedText === text : text.includes(expectedText);
    });
}
export const expectPriorInput = (selector, expectedText) => (app) => __awaiter(void 0, void 0, void 0, function* () {
    let idx = 0;
    yield app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const inputText = yield app.client.$(selector).then(_ => _.getText());
        if (++idx > 5) {
            console.error(`still waiting for prior input actual=${inputText} expected=${expectedText}`);
        }
        return inputText === expectedText;
    }), waitTimeoutOption);
    return app;
});
/** @return the "N" of the current block */
export function nOfCurrentBlock(app, splitIndex = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const attr = yield app.client
            .$(Selectors.CURRENT_PROMPT_BLOCK_FOR_SPLIT(splitIndex))
            .then(_ => _.getAttribute(Selectors.N_ATTR));
        return parseInt(attr, 10);
    });
}
/** Index of last executed block */
export function lastBlock(app, splitIndex = 1, N = 1, inNotebook = false) {
    return __awaiter(this, void 0, void 0, function* () {
        return {
            app,
            splitIndex: splitIndex,
            count: parseInt(yield app.client
                .$(Selectors.PROMPT_BLOCK_LAST_FOR_SPLIT(splitIndex, N, inNotebook))
                .then(_ => _.getAttribute(Selectors.N_ATTR)), 10)
        };
    });
}
/** Click to expand the last replayed sample output */
export function expandNth(app, N, splitIndex = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const expando = yield app.client.$(Selectors.EXPANDABLE_OUTPUT_N(N, splitIndex));
        yield expando.waitForExist(waitTimeoutOption);
        yield expando.click();
    });
}
/** Click to expand the last replayed sample output */
export function expandLast(app, splitIndex = 1, N = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        const expando = yield app.client.$(Selectors.EXPANDABLE_OUTPUT_LAST_IN_NOTEBOOK(splitIndex, N));
        yield expando.waitForExist(waitTimeoutOption);
        yield expando.click();
    });
}
//# sourceMappingURL=cli.js.map